@@ -26,6 +26,12 @@ using org.freedesktop.DBus;
 
 namespace Imendio.Blam {
 
+	public enum TargetType {
+		String,
+		UriList,
+		Channel
+	};
+
 #if ENABLE_DBUS
     [NDesk.DBus.Interface("org.gnome.feed.Reader")]
 #endif
@@ -89,6 +95,7 @@ namespace Imendio.Blam {
 
         private AddChannelDialog  addChannelDialog;
         private ChannelDialog     channelDialog;
+        private AddGroupDialog    addGroupDialog;
         private PreferencesDialog preferencesDialog;
         private OpmlDialog        opmlDialog;
 
@@ -98,12 +105,6 @@ namespace Imendio.Blam {
 
         Gnome.Client client = null;
 
-
-        enum TargetType {
-            String,
-            UriList
-        };
-
         public static TargetEntry[] DragEntries = new TargetEntry[] {
             new TargetEntry("STRING", 0, (uint)TargetType.String),
             new TargetEntry("text/plain", 0, (uint)TargetType.String),
@@ -128,6 +129,12 @@ namespace Imendio.Blam {
             }
         }
 
+        public ChannelList ChannelList {
+            get {
+                return channelList;
+            }
+        }
+
 		public ThemeManager ThemeManager {
 			get {
 				return mThemeManager;
@@ -156,6 +163,7 @@ namespace Imendio.Blam {
 
             mCollection.ChannelUpdated         += ChannelUpdatedCb;
             mCollection.ChannelAdded           += ChannelAddedCb;
+			mCollection.ChannelGroupAdded      += ChannelGroupAddedCb;
             mCollection.ChannelRemoved         += ChannelRemovedCb;
             mCollection.ChannelRefreshStarted  += ChannelRefreshStartedCb;
             mCollection.ChannelRefreshFinished += ChannelRefreshFinishedCb;
@@ -179,7 +187,7 @@ namespace Imendio.Blam {
                                                         "mainWindow", null);
             gladeXML.Autoconnect(this);
 
-            channelList = new ChannelList(mCollection.Channels);
+            channelList = new ChannelList(mCollection.Channels, mCollection.Groups);
             ((Container)channelListSw).Child = channelList;
 
             channelList.ChannelSelectedEvent   += ChannelSelected;
@@ -205,6 +213,11 @@ namespace Imendio.Blam {
 
             trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"));
             trayIcon.ButtonPressEvent += TrayIconButtonPressCb;
+            trayIcon.RefreshAllEvent += RefreshAllActivated;
+            trayIcon.PreferencesEvent += PreferencesActivated;
+            trayIcon.AboutEvent += AboutActivated;
+            trayIcon.QuitEvent += QuitActivated;
+
             channelsLabelText = channelsLabel.Text;
             UpdateTotalNumberOfUnread ();
 
@@ -231,6 +244,7 @@ namespace Imendio.Blam {
 
             channelDialog = new ChannelDialog (this);
             addChannelDialog = new AddChannelDialog (this);
+            addGroupDialog = new AddGroupDialog (this);
             preferencesDialog = new PreferencesDialog (this.Window);
             opmlDialog = new OpmlDialog (this.Window);
             opmlDialog.ChannelAdded += mCollection.Add;
@@ -316,44 +330,7 @@ namespace Imendio.Blam {
         }
 #endif
 
-#if FALSE
-        private void OnButtonPressEvent(object o, Gecko.DomMouseClickArgs args)
-        {
-            int button = Utils.GeckoGetMouseButton(args.DomEvent);
-
-            if(button == 2){ // Right button
-                Gtk.Menu popup = new Gtk.Menu();
-
-                if(itemView.Widget.LinkMessage != null && itemView.Widget.LinkMessage != string.Empty){ // Make sure we're on a link.
-                    GtkUtil.AppendMenuItem(popup,
-                                       Catalog.GetString("_Open link in browser"),
-                                       new EventHandler(OpenLink), true);
-                    GtkUtil.AppendMenuItem(popup,
-                                       Catalog.GetString("_Copy link location"),
-                                       "gtk-copy", new EventHandler(CopyLink), true);
-
-                    popup.Popup();
-                }
-            }
-
-            args.RetVal = 0;
-        }
-#endif
-
-        void CopyLink(object o, EventArgs args)
-        {
-            Clipboard clip = Clipboard.Get(Gdk.Atom.Intern("CLIPBOARD", true));
-						// FIXME: WebKit
-            //clip.Text = itemView.Widget.LinkMessage;
-        }
-
-        void OpenLink(object o, EventArgs args)
-        {
-						// FIXME: WebKit
-            //Gnome.Url.Show(itemView.Widget.LinkMessage);
-        }
-
-        private void ChannelSelected(Channel channel)
+        private void ChannelSelected(IChannel channel)
         {
             if (channel == null) {
                 SensitizeChannelMenuItems(false);
@@ -406,7 +383,7 @@ namespace Imendio.Blam {
 
         public void MenuChannelActivated (object obj, EventArgs args)
         {
-            Channel channel = channelList.GetSelected ();
+            IChannel channel = channelList.GetSelected ();
 
             bool sensitive = true;
             if (channel == null) {
@@ -597,10 +574,23 @@ namespace Imendio.Blam {
             addChannelDialog.Show ();
         }
 
-        public void EditChannelActivated (Channel channel)
+        public void AddGroupActivated (object obj, EventArgs args)
         {
-            if (channel != null) {
-                channelDialog.Show (channel);
+            addGroupDialog.Show();
+        }
+
+        public void EditChannelActivated (IChannel channel)
+        {
+            bool IsGroup = false;
+
+            foreach(ChannelGroup group in mCollection.Groups){
+                if(channel == group){
+                    IsGroup = true;
+                }
+            }
+
+            if (channel != null && !IsGroup) {
+                channelDialog.Show (channel as Channel);
             }
         }
 
@@ -609,7 +599,7 @@ namespace Imendio.Blam {
             EditChannelActivated (channelList.GetSelected ());
         }
 
-        public void MarkChannelAsReadActivated (Channel channel)
+        public void MarkChannelAsReadActivated (IChannel channel)
         {
             if (channel.MarkAsRead ()) {
                 CCollection.Update (channel);
@@ -628,7 +618,7 @@ namespace Imendio.Blam {
          }
         }
 
-        public void RemoveChannelActivated(Channel channel)
+        public void RemoveChannelActivated(IChannel channel)
         {
             if (channel != null) {
                 RemoveChannelDialog.Show (mainWindow, mCollection, channel);
@@ -637,12 +627,12 @@ namespace Imendio.Blam {
 
         public void RemoveChannelActivated(object obj, EventArgs args)
         {
-            Channel channel = channelList.GetSelected ();
+            IChannel channel = channelList.GetSelected ();
 
             RemoveChannelActivated(channel);
         }
 
-        public void RefreshChannelActivated(Channel channel)
+        public void RefreshChannelActivated(IChannel channel)
         {
             if (channel != null) {
                 mCollection.Refresh (channel);
@@ -651,7 +641,7 @@ namespace Imendio.Blam {
 
         public void RefreshChannelActivated(object obj, EventArgs args)
         {
-            Channel channel = channelList.GetSelected ();
+            IChannel channel = channelList.GetSelected ();
 
             RefreshChannelActivated(channel);
         }
@@ -688,24 +678,29 @@ namespace Imendio.Blam {
             }
         }
 
-        private void ChannelAddedCb (Channel channel)
+        private void ChannelAddedCb (IChannel channel)
         {
             channelList.Add (channel);
         }
 
-        private void ChannelUpdatedCb (Channel channel)
+		private void ChannelGroupAddedCb(IChannel group, IChannel chan)
+		{
+			channelList.AddToGroup(group, chan);
+		}
+
+        private void ChannelUpdatedCb (IChannel channel)
         {
             channelList.Updated (channel);
             UpdateTotalNumberOfUnread ();
         }
 
-        private void ChannelRemovedCb (Channel channel)
+        private void ChannelRemovedCb (IChannel channel)
         {
             channelList.Remove (channel);
             UpdateTotalNumberOfUnread ();
         }
 
-        private void ChannelRefreshStartedCb (Channel channel)
+        private void ChannelRefreshStartedCb (IChannel channel)
         {
             uint contextId = statusbar.GetContextId("update-status");
             string statusString;
@@ -715,7 +710,7 @@ namespace Imendio.Blam {
             statusbar.Push (contextId, statusString);
         }
 
-        private void ChannelRefreshFinishedCb (Channel channel)
+        private void ChannelRefreshFinishedCb (IChannel channel)
         {
             uint contextId = statusbar.GetContextId("update-status");
 
@@ -781,7 +776,12 @@ namespace Imendio.Blam {
 
         private void TrayIconButtonPressCb (object o, EventArgs args)
         {
-            mainWindow.Present ();
+            /* If we're hidden, show and if we're in fg, hide. */
+            if(mainWindow.HasToplevelFocus){
+                mainWindow.Hide();
+            } else {
+                mainWindow.Present();
+            }
         }
 
         private void RestoreWindowState()
@@ -854,11 +854,6 @@ namespace Imendio.Blam {
 
             trayIcon.Tooltip = string.Format (Catalog.GetPluralString ("{0} unread item", "{0} unread items", nrOfUnread),
                                              nrOfUnread);
-            if(nrOfUnread > 0 || !mainWindow.Visible){
-                trayIcon.Show();
-            } else {
-                trayIcon.Hide();
-            }
         }
 
 	public static void SetProcessName(string name)
