@@ -18,6 +18,7 @@ namespace Imendio.Blam {
     public class ChannelCollection {
 
 	public event ChannelEventHandler ChannelAdded;
+	public event ChannelGroupEventHandler ChannelGroupAdded;
 	public event ChannelEventHandler ChannelUpdated;
 	public event ChannelEventHandler ChannelRemoved;
 	
@@ -46,6 +47,8 @@ namespace Imendio.Blam {
 	    }
 	}
 
+	[XmlElement("Group", typeof(ChannelGroup))] public ArrayList Groups;
+
 	public int NrOfUnreadItems {
 	    get {
 		int unread = 0;
@@ -53,6 +56,10 @@ namespace Imendio.Blam {
 		foreach (Channel channel in mChannels) {
 		    unread += channel.NrOfUnreadItems;
 		}
+
+        foreach (ChannelGroup changrp in Groups) {
+            unread += changrp.NrOfUnreadItems;
+        }
 		
 		return unread;
 	    }
@@ -130,13 +137,9 @@ namespace Imendio.Blam {
 
 		// Move the file to the real one
 		try {
-		    if (File.Exists (this.FileName)) {
-			File.Delete (this.FileName);
-		    }
-
-		    File.Move (tmpFile, this.FileName);
+		    File.Replace (tmpFile, this.FileName, this.FileName + ".bk");
 		} catch (Exception e) {
-		    Console.Error.WriteLine ("Move failed: " + e.Message);
+		    Console.Error.WriteLine ("File replace error: " + e.Message);
 		    return;
 		}
 		
@@ -144,7 +147,7 @@ namespace Imendio.Blam {
 	    }
 	}
 
-	public void Add (Channel channel)
+	public void Add (IChannel channel)
 	{
 	    // Not the most efficient way of doing things :)
 	    foreach (Channel ch in mChannels) {
@@ -153,7 +156,10 @@ namespace Imendio.Blam {
 		}
 	    }
 
-        channel.Name = channel.Url;
+        if(channel.Name == null){
+            channel.Name = channel.Url;
+        }
+
         mChannels.Add (channel);
         Refresh (channel);
 
@@ -164,7 +170,19 @@ namespace Imendio.Blam {
         MarkAsDirty(true);
 	}
 
-	public void Update (Channel channel)
+	public void Add(ChannelGroup group, IChannel channel)
+	{
+		group.Add(channel);
+		Refresh(channel);
+
+		if(ChannelGroupAdded != null){
+			ChannelGroupAdded(group, channel);
+		}
+
+		MarkAsDirty(true);
+	}
+
+	public void Update (IChannel channel)
 	{
 	    MarkAsDirty (true);
 
@@ -173,9 +191,22 @@ namespace Imendio.Blam {
 	    }
 	}
 
-	public void Remove (Channel channel)
+	public void Remove (IChannel channel)
 	{
-	    mChannels.Remove (channel);
+        /* Try to find out from which list we need to remove. */
+        if(mChannels.Contains(channel)){
+            mChannels.Remove (channel);
+        } else if(Groups.Contains(channel)){
+            Groups.Remove(channel);
+        } else {
+            /* It's not a first-level channel or group. Dig deeper. */
+            foreach(ChannelGroup group in Groups){
+                if(group.Channels.Contains(channel)){
+                    group.Channels.Remove(channel);
+                    break;
+                }
+            }
+        }
 
 	    if (ChannelRemoved != null) {
 		ChannelRemoved (channel);
@@ -184,7 +215,7 @@ namespace Imendio.Blam {
         MarkAsDirty(true);
 	}
 
-	public void Refresh (Channel channel) 
+	public void Refresh (IChannel channel)
 	{
 	    mQueue.Enqueue (channel);
 	    EmitChannelRefreshStarted (channel);
@@ -195,7 +226,7 @@ namespace Imendio.Blam {
 	    thread.Start ();
 	}
 
-	private void QueueChannelRefresh (Channel channel)
+	private void QueueChannelRefresh (IChannel channel)
 	{
 	    mQueue.Enqueue (channel);
 	    EmitChannelRefreshStarted (channel);
@@ -230,6 +261,13 @@ namespace Imendio.Blam {
 		nrOfChannels++;
 	    }
 
+        foreach(ChannelGroup group in Groups){
+            foreach(Channel channel in group.Channels){
+                QueueChannelRefresh(channel);
+                nrOfChannels++;
+            }
+        }
+
 	    StartRefreshThreads (nrOfChannels);
 	}
 
@@ -245,7 +283,19 @@ namespace Imendio.Blam {
 		    nrOfChannels++;
 		}
 	    }
-	    
+
+        foreach(ChannelGroup group in Groups){
+            if(group.Channels.Count == 0)
+                    continue;
+            foreach(Channel channel in group.Channels){
+                TimeSpan span = DateTime.Now.Subtract(channel.LastRefreshed);
+                if(span.TotalSeconds >= refreshRate * 60){
+                    QueueChannelRefresh(channel);
+                    nrOfChannels++;
+                }
+            }
+        }
+
 	    StartRefreshThreads (nrOfChannels);
 	}
 	
@@ -257,8 +307,26 @@ namespace Imendio.Blam {
 		    ch.MarkItemIdAsRead (id);
 		}
 	    }
+        foreach (ChannelGroup gr in Groups) {
+            foreach(Channel chan in gr.Channels){
+                if(chan != channel){
+                    chan.MarkItemIdAsRead(id);
+                }
+            }
+        }
 	}
 
+    public bool CheckItemIdReadInAllChannels(string id)
+    {
+        foreach(IChannel ch in mChannels){
+            Item item = ch.GetItem(id);
+            if(item != null){
+                return true;
+            }
+        }
+        return false;
+    }
+
 	private void UpdateThread ()
 	{
 	    while (true) {
@@ -279,7 +347,7 @@ namespace Imendio.Blam {
 	    mRunningList.Remove (Thread.CurrentThread);
 	}
 
-	private void EmitChannelRefreshStarted (Channel channel)
+	private void EmitChannelRefreshStarted (IChannel channel)
 	{
 	    if (ChannelRefreshStarted != null) {
 		ChannelRefreshStarted (channel);
