@@ -33,8 +33,15 @@ namespace Imendio.Blam {
         private TreeViewColumn nameColumn;
 
         private IEnumerator channelEnumerator;
+        private IEnumerator groupEnumerator;
 
-        public ChannelList(IList channels)
+        private IChannel LastChannel = null;
+
+        public static TargetEntry[] DragEntries = new TargetEntry[] {
+            new TargetEntry("channel", TargetFlags.Widget, (uint)TargetType.Channel)
+       };
+
+        public ChannelList(IList channels, IList groups)
         { 
             TreeViewColumn col;
             CellRenderer   cell;
@@ -65,12 +72,12 @@ namespace Imendio.Blam {
             selectionChangedHandler = new EventHandler(SelectionChanged);
             this.Selection.Changed += selectionChangedHandler;
 
-            this.Model = new ListStore (typeof(Channel));
+            this.Model = new TreeStore (typeof(Channel));
             this.HeadersVisible = false;
 
             // Sort the list
-            (Model as ListStore).DefaultSortFunc = ChannelSort;
-            ((ListStore)this.Model).SetSortColumnId (-1, SortType.Ascending);
+            (Model as TreeStore).DefaultSortFunc = ChannelSort;
+            (this.Model as TreeStore).SetSortColumnId (-1, SortType.Ascending);
 
             // Right click popup
             this.popupMenu = new ChannelMenu();
@@ -80,32 +87,140 @@ namespace Imendio.Blam {
             this.popupMenu.RefreshSelected    += RefreshChannelCb;
 
             channelEnumerator = channels.GetEnumerator();
+            groupEnumerator = groups.GetEnumerator();
+
+			EnableModelDragSource(ModifierType.Button1Mask, DragEntries, DragAction.Copy);
+			DragDataGet += DragDataGetHandler;
+			EnableModelDragDest(DragEntries, DragAction.Copy);
+			DragDataReceived += DragDataReceivedHandler;
 
             GLib.Idle.Add(new GLib.IdleHandler(IdleAdd));
         }
 
         private void ForceResort()
         {
-            (Model as ListStore).SetSortColumnId(-1, SortType.Descending);
-            (Model as ListStore).SetSortColumnId(-1, SortType.Ascending);
+            (Model as TreeStore).SetSortColumnId(-1, SortType.Descending);
+            (Model as TreeStore).SetSortColumnId(-1, SortType.Ascending);
         }
 
+		private void DragDataGetHandler(object o, DragDataGetArgs args)
+		{
+			ChannelList chlst = o as ChannelList;
+			args.RetVal = chlst.GetSelected();
+
+		}
+
+		private void DragDataReceivedHandler(object o, DragDataReceivedArgs args)
+		{
+			TreePath path;
+			TreeViewDropPosition pos;
+			TreeIter iter, tmp_iter;
+			IChannel src = GetSelected();
+			tmp_iter = src.Iter;
+			GetDestRowAtPos(args.X, args.Y, out path, out pos);
+			Model.GetIter(out iter, path);
+			IChannel dst = Model.GetValue(iter, 0) as IChannel;
+
+			/* Not directly over a channel. */
+			if(pos == TreeViewDropPosition.After || pos == TreeViewDropPosition.Before){
+
+				TreePath tmp_path = path.Copy();
+				tmp_path.Up();
+				TreePath tmp_path_src = Model.GetPath(src.Iter).Copy();
+				tmp_path_src.Up();
+				if(tmp_path == tmp_path_src){ // Same parent.
+					System.Console.WriteLine("return!");
+					return;
+				}
+
+				Blam.Application.TheApp.CCollection.Remove(src);
+
+				Model.GetIter(out tmp_iter, tmp_path);
+				IChannel drop_parent = Model.GetValue(tmp_iter, 0) as IChannel;
+				//System.Console.WriteLine("dp: {0} depth:{1}", drop_parent.GetType(), path.Depth);
+				if(path.Depth == 2 && drop_parent is ChannelGroup){ // Dropped inside a group.
+					Blam.Application.TheApp.CCollection.Add(drop_parent as ChannelGroup, src);
+				} else{
+					Blam.Application.TheApp.CCollection.Add(src);
+				}
+			/* Directly over a channel. */
+			} else {
+				TreePath tmp_path_src = Model.GetPath(src.Iter).Copy();
+				tmp_path_src.Up();
+				TreePath dst_path_parent = path.Copy();
+				dst_path_parent.Up();
+
+				if(dst is Channel && path.Depth == 1 && // dst is top-level
+				   src is Channel && Model.GetPath(src.Iter).Depth == 2){ // src is in a group
+					Blam.Application.TheApp.CCollection.Remove(src);
+					Blam.Application.TheApp.CCollection.Add(src);
+				}
+
+				/*
+				 * A channel can't become a group and a group can't be
+				 * inside another group and it doesn't make sense to move
+				 * it into the same group.
+				 * FIXME: Make it possible for a group to be inside another group.
+				 */
+				if((dst is Channel && path.Depth == 1) || src is ChannelGroup ||
+				   path == tmp_path_src){ // Move to the same group
+					return;
+				}
+
+				Blam.Application.TheApp.CCollection.Remove(src);
+				/*
+				 * If we drop on a channel which is inside a group, we add that channel to
+				 * the group.
+				 */
+
+				if(dst is Channel && path.Depth == 2){ // Channel inside a group.
+					Model.GetIter(out tmp_iter, dst_path_parent);
+					dst = Model.GetValue(tmp_iter, 0) as ChannelGroup;
+				}
+
+				Blam.Application.TheApp.CCollection.Add(dst as ChannelGroup, src);
+			}
+		}
+
         private bool IdleAdd()
         {
             while (channelEnumerator.MoveNext ()) {
-                Channel channel = (Channel)channelEnumerator.Current;
+                IChannel channel = channelEnumerator.Current as IChannel;
                 Add(channel);
             }
 
+            while(groupEnumerator.MoveNext()){
+                IChannel group = groupEnumerator.Current as IChannel;
+                AddGroup(group);
+            }
+
             return false;
         }
 
-        public void Add (Channel channel)
+        public void Add (IChannel channel)
         {
-            ((ListStore)this.Model).AppendValues(channel);
+            channel.Iter = (this.Model as TreeStore).AppendValues(channel);
         }
 
-        public Channel GetSelected ()
+		public void AddToGroup(IChannel group, IChannel channel)
+		{
+			channel.Iter = (Model as TreeStore).AppendValues(group.Iter, channel);
+		}
+
+        public void AddGroup(IChannel channel)
+        {
+            TreeIter iter = (this.Model as TreeStore).AppendValues(channel);
+            ChannelGroup group = channel as ChannelGroup;
+            group.Iter = iter;
+			SetDragDestRow(Model.GetPath(iter), TreeViewDropPosition.IntoOrAfter);
+
+            foreach(IChannel chan in group.Channels){
+                iter = (this.Model as TreeStore).AppendValues(group.Iter, chan);
+                chan.Iter = iter;
+            }
+        }
+
+        public IChannel GetSelected ()
         {
             TreeModel model;
             TreeIter  iter;
@@ -114,10 +229,10 @@ namespace Imendio.Blam {
                 return null;
             }
 
-            return(Channel)model.GetValue(iter, 0);
+            return(IChannel)model.GetValue(iter, 0);
         }
 
-        public void Updated (Channel channel)
+        public void Updated (IChannel channel)
         {
             TreeIter iter = FindChannel (channel);
 
@@ -129,7 +244,7 @@ namespace Imendio.Blam {
 
         }
 
-        public void Remove (Channel channel)
+        public void Remove (IChannel channel)
         {
             TreeIter iter = FindChannel(channel);
             bool valid;
@@ -137,7 +252,7 @@ namespace Imendio.Blam {
             if (!iter.Equals(TreeIter.Zero)) {
                 this.Selection.Changed -= selectionChangedHandler;
 
-                valid = ((ListStore)this.Model).Remove(ref iter);
+                valid = (this.Model as TreeStore).Remove(ref iter);
 
                 this.Selection.Changed += selectionChangedHandler;
 
@@ -156,7 +271,7 @@ namespace Imendio.Blam {
             TreeIter  iter;
 
             if (this.Selection.GetSelected(out model, out iter)) {
-                Channel channel = (Channel)model.GetValue(iter, 0);
+                IChannel channel = (IChannel)model.GetValue(iter, 0);
                 if (channel.NrOfUnreadItems > 0) {
                     return true;
                 }
@@ -175,7 +290,7 @@ namespace Imendio.Blam {
             TreeIter startIter = iter;
 
             do {
-                Channel channel = (Channel)model.GetValue(iter, 0);
+                IChannel channel = model.GetValue(iter, 0) as IChannel;
                 if (channel.NrOfUnreadItems > 0) {
                     this.Selection.SelectIter(iter);
                     ScrollToCell(this.Model.GetPath(iter), nameColumn,
@@ -219,8 +334,8 @@ namespace Imendio.Blam {
 
         private int ChannelSort(TreeModel model, TreeIter ia, TreeIter ib)
         {
-            Channel a = Model.GetValue(ia, 0) as Channel;
-            Channel b = Model.GetValue(ib, 0) as Channel;
+            IChannel a = Model.GetValue(ia, 0) as IChannel;
+            IChannel b = Model.GetValue(ib, 0) as IChannel;
 
             return a.Name.CompareTo(b.Name);
         }
@@ -259,14 +374,14 @@ namespace Imendio.Blam {
             TreeSelection selection = (TreeSelection)obj;
             TreeIter      iter;
             TreeModel     model;
-            Channel       channel;
+            IChannel      channel;
 
             if (!selection.GetSelected(out model, out iter)) {
                 EmitChannelSelected(null);
                 return;
             }
 
-            channel = (Channel)model.GetValue(iter, 0);
+            channel = model.GetValue(iter, 0) as IChannel;
             if (channel != null) {
                 EmitChannelSelected(channel);
             }
@@ -274,7 +389,7 @@ namespace Imendio.Blam {
 
         private void NamesCellDataFunc(TreeViewColumn col, CellRenderer cell, TreeModel model, TreeIter iter)
         {
-            Channel channel = model.GetValue(iter, 0) as Channel;
+            IChannel channel = model.GetValue(iter, 0) as IChannel;
             int weight = (int)Pango.Weight.Normal;
 
             (cell as CellRendererText).Text = channel.Name;
@@ -291,7 +406,7 @@ namespace Imendio.Blam {
                                        TreeModel      model,
                                        TreeIter       iter)
         {
-            Channel channel = (Channel)model.GetValue(iter, 0);
+            IChannel channel = model.GetValue(iter, 0) as IChannel;
             int weight = (int)Pango.Weight.Normal;
 
             ((CellRendererText)cell).Text = channel.NrOfUnreadItems + "/" + channel.NrOfItems;
@@ -303,21 +418,29 @@ namespace Imendio.Blam {
             ((CellRendererText)cell).Weight = weight;
         }
 
-        private void EmitChannelSelected(Channel channel)
+        private void EmitChannelSelected(IChannel channel)
         {
+            if(LastChannel != null){
+                foreach(Item item in LastChannel.Items){
+                    item.Old = true;
+                }
+            }
+
+            LastChannel = channel;
+
             if (ChannelSelectedEvent != null) {
                 ChannelSelectedEvent(channel);
             }
         }
 
         // Used by ChannelUpdated
-        private Channel findChannel;
+        private IChannel findChannel;
         private TreeIter foundIter;
         private bool ForeachFindChannel(TreeModel model, 
                                         TreePath  path,
                                         TreeIter  iter)
         {
-            Channel channel = (Channel)model.GetValue(iter, 0);
+            IChannel channel = model.GetValue(iter, 0) as IChannel;
             if (channel == findChannel) {
                 foundIter = iter;
                 return true;
@@ -326,7 +449,7 @@ namespace Imendio.Blam {
             return false;
         }
 
-        private TreeIter FindChannel(Channel channel) 
+        private TreeIter FindChannel(IChannel channel)
         {
             findChannel = channel;
             foundIter = TreeIter.Zero;
@@ -336,7 +459,7 @@ namespace Imendio.Blam {
             return foundIter;
         }
 
-        private void EmitEditChannelEvent(Channel channel)
+        private void EmitEditChannelEvent(IChannel channel)
         {
             if (EditChannelEvent != null) {
                 EditChannelEvent(channel);
@@ -359,20 +482,20 @@ namespace Imendio.Blam {
             GtkUtil.AppendMenuItem (popup_menu, 
                                     Catalog.GetString ("_Mark as read"),
                                     new EventHandler (EmitMarkAsReadCb));
-            GtkUtil.AppendMenuItem (popup_menu, 
+            GtkUtil.AppendMenuItem (popup_menu,
                                     Catalog.GetString ("_Refresh"),
-                                    "gtk-refresh",
+                                    Stock.Refresh,
                                     new EventHandler (EmitRefreshChannelCb),
                                     true);
             GtkUtil.AppendMenuSeparator (popup_menu);
             GtkUtil.AppendMenuItem (popup_menu,
                                     Catalog.GetString ("_Edit"),
-                                    "gtk-edit",
+                                    Stock.Edit,
                                     new EventHandler (EmitEditCb),
                                     true);
             GtkUtil.AppendMenuItem (popup_menu,
                                     Catalog.GetString ("Remo_ve"),
-                                    "gtk-remove",
+                                    Stock.Remove,
                                     new EventHandler (EmitRemoveCb), true);
             popup_menu.Popup (null, null, null, 
                               eb.Button, eb.Time);
