@@ -22,13 +22,14 @@ namespace Imendio.Blam {
 
 	private ItemView itemView;
 	
-	private TreeViewColumn titleColumn; 
+    private TreeViewColumn titleColumn;
+    private TreeViewColumn iconColumn;
 
     private Item lastItem;
     private uint lastTimeout;
 
-	private Channel channel;
-	public Channel CurrentChannel {
+	private IChannel channel;
+	public IChannel CurrentChannel {
 	    get {
 		return channel;
 	    }
@@ -42,6 +43,17 @@ namespace Imendio.Blam {
 	public ItemList(ItemView itemView) 
 	{
 	    this.itemView = itemView;
+
+        CellRendererPixbuf cell2 = new CellRendererPixbuf();
+        iconColumn = new TreeViewColumn();
+        iconColumn.PackStart(cell2, true);
+        iconColumn  .Sizing = TreeViewColumnSizing.Autosize;
+        iconColumn.Expand = false;
+        iconColumn.SetCellDataFunc(cell2,
+                   new TreeCellDataFunc(IconCellDataFunc));
+
+        AppendColumn(iconColumn);
+
 	    titleColumn = new TreeViewColumn();
 	    CellRendererText cell = new CellRendererText();
 
@@ -265,6 +277,29 @@ namespace Imendio.Blam {
         ((CellRendererText)cell).Weight = weight;
         ((CellRendererText)cell).Ellipsize = Pango.EllipsizeMode.End;
 	}
+
+    private void IconCellDataFunc(TreeViewColumn col,
+                       CellRenderer   cell,
+				       TreeModel      model,
+				       TreeIter       iter)
+    {
+        Item item = model.GetValue(iter, 0) as Imendio.Blam.Item;
+        string icon = null;
+
+        if(item.Unread == true){
+            icon = "stock_book_red";
+        } else {
+            icon = "stock_book_open";
+        }
+
+        if(item.Old == true && item.Unread == true){
+            icon = "stock_book_green";
+        }
+
+        if((cell as CellRendererPixbuf).IconName != icon){
+            (cell as CellRendererPixbuf).IconName = icon;
+        }
+    }
 	
 	private void EmitItemSelected (Imendio.Blam.Item item) 
 	{
