@@ -14,6 +14,7 @@ using System.Collections;
 namespace Imendio.Blam {
     public class Preference {
         public static string REVERSE_ENTRIES   = "behaviour/reverse_entries";
+        public static string REFRESH_AT_START  = "behaviour/refresh_at_start";
         public static string AUTO_REFRESH      = "behaviour/auto_refresh";
         public static string AUTO_REFRESH_RATE = "behaviour/auto_refresh_rate";
         public static string IGNORE_SSL_ERR    = "behaviour/ignore_ssl_err";
@@ -32,6 +33,7 @@ namespace Imendio.Blam {
         [Widget] Gtk.CheckButton reverseEntriesCheckButton = null;
         [Widget] Gtk.CheckButton autoRefreshCheckButton    = null;
         [Widget] Gtk.CheckButton ignoreSSLErr              = null;
+        [Widget] Gtk.CheckButton RefreshOnStartup          = null;
         [Widget] Gtk.SpinButton  refreshRateSpinButton     = null;
         [Widget] Gtk.Label       boldThemeLabel            = null;
         [Widget] Gtk.ComboBox    themeComboBox             = null;
@@ -57,6 +59,10 @@ namespace Imendio.Blam {
 			int refreshRate = Conf.Get (Preference.AUTO_REFRESH_RATE, 15);
 			refreshRateSpinButton.Value = refreshRate;
 
+            ignoreSSLErr.Active = Conf.Get(Preference.IGNORE_SSL_ERR, false);
+
+            RefreshOnStartup.Active = Conf.Get(Preference.REFRESH_AT_START, false);
+
 			boldThemeLabel.Markup = "<b>" + boldThemeLabel.Text + "</b>";
 
 			themeComboBox.Changed += new EventHandler(ThemeSelectionChanged);
@@ -116,6 +122,11 @@ namespace Imendio.Blam {
 		               (bool) ignoreSSLErr.Active);
 		}
 
+        public void RefreshOnStartupToggled(object o, EventArgs args)
+        {
+            Conf.Set(Preference.REFRESH_AT_START, (bool)RefreshOnStartup.Active);
+        }
+
 		public void ThemeSelectionChanged (object obj, EventArgs args)
 		{
 			Gtk.ComboBox box = obj as Gtk.ComboBox;
